package br.com.inpe.ip.application.mb;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.bean.ViewScoped;
import javax.faces.model.SelectItem;
import javax.inject.Inject;
import javax.inject.Named;

import org.hibernate.exception.ConstraintViolationException;

import br.com.inpe.ip.domain.entity.Employee;
import br.com.inpe.ip.domain.entity.Group;
import br.com.inpe.ip.domain.entity.StaffType;
import br.com.inpe.ip.domain.service.EmployeeService;
import br.com.inpe.ip.domain.service.GroupService;
import br.com.inpe.ip.infrastructure.dao.exception.BusinessException;

@Named
@ViewScoped
public class EmployeeMB extends BaseBean implements Serializable {

	@Inject private EmployeeService employeeService;
	@Inject private GroupService groupService;

	private Employee employee = new Employee();

	private List<Employee> employees;
	private List<Group> groups;
	private List<SelectItem> employeesItems;

	public void saveOrUpdate () {
		try {									
			if ( employee.getId() == null ) {
				employeeService.save(employee);
				employee = new Employee();
				addInfoMessage("funcionário cadastrado com sucesso!");
			} else {
				employeeService.update(employee);
				employee = new Employee();
				addInfoMessage("funcionário alterado com sucesso!");	
			}
		} catch (ConstraintViolationException cve) {
			addErrorMessage("funcionário já existe");
		} catch ( Exception e ) {
			e.printStackTrace();			
			addErrorMessage("ocorreu um erro inexperado!");
		}
	} 

	public void remove () {
		try {
			employeeService.remove(employee);		
			employee = new Employee();
			employees = employeeService.findAll();
		} catch (Exception e) {
			addErrorMessage(e.getMessage());
		}
	}

	public List<Group> completeGroupBy (String name) {
		try {
			if ( groups == null ) {
				groups = new ArrayList<Group>();
			}
			return groupService.searchBy(name);		
		} catch (BusinessException be) {
			addInfoMessage(be.getMessage());
		} catch (Exception e) {
			addErrorMessage(e.getMessage());
			e.printStackTrace();
		}
		return groups = new ArrayList<Group>();
	}

	public List<SelectItem> getEmployeesItems() {
		if (employeesItems == null) {
			employeesItems = new ArrayList<SelectItem>();
			employeesItems.add(new SelectItem(null, ""));
			for (StaffType s : StaffType.values()) {
				employeesItems.add(new SelectItem(s, s.getDescription()));				
			}
		}
		return employeesItems;
	}
	
	public Employee getEmployee() {
		return employee;
	}

	public void setEmployee(Employee employee) {
		this.employee = employee;
	}

	public List<Employee> getEmployees() {		
		return employees == null ? employees = employeeService.findAll() : employees;
	}

	public void setEmployees(List<Employee> employees) {
		this.employees = employees;
	}

	public List<Group> getGroups() {
		return groups;
	}

	public void setGroups(List<Group> groups) {
		this.groups = groups;
	}

	private static final long serialVersionUID = 3685462659015340980L;

}
