package br.com.inpe.ip.domain.entity;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotBlank;

@Entity
@Table(name = "EMPLOYEE", schema = "ip", uniqueConstraints = {
		@UniqueConstraint(columnNames = {"NAME"})
})
public class Employee implements Serializable {	
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "EMPLOYEE_ID")
	private Integer id;
	
	@NotBlank(message = "nome é campo obrigatório!")
	@Column(name = "NAME", nullable = false, length = 100)
	private String name;
	
	@Transient
	private StaffType staffType;
		
	@Column(name = "STAFF_TYPE", nullable = false, length = 100)
	private String staffType_;
	
	@Email(message = "e-mail inválido!")
	@Column(name = "EMAIL", nullable = false, length = 50)
	private String email;
		
	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST})
	@JoinColumn(name = "GROUP_ID", referencedColumnName = "GROUP_ID")	
	private Group group;
	
	@NotBlank(message = "sala é campo obrigatório!")
	@Column(name = "ROOM_NUMBER", nullable = false, length = 20)
	private String room;
	
	@NotBlank(message = "ramal é campo obrigatório!")
	@Column(name = "BRANCH", nullable = false, length = 20)
	private String branch;
	
	@NotBlank(message = "IP é campo obrigatório!")
	@Column(name = "IP", nullable = false, length = 20)
	private String ip;
	
	public Employee () { 
		group = new Group();
	}				
	
	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}		

	public StaffType getStaffType() { 
		for (StaffType s : StaffType.values())
			if (s.getCode().equals(staffType_))
				return staffType = s;
		return staffType;
	}

	public void setStaffType(StaffType staffType) {
		this.staffType_ = staffType.getCode();
		this.staffType = staffType;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}
	
	public Group getGroup() {
		return group;
	}

	public void setGroup(Group group) {
		this.group = group;
	}

	public String getRoom() {
		return room;
	}

	public void setRoom(String room) {
		this.room = room;
	}

	public String getBranch() {
		return branch;
	}

	public void setBranch(String branch) {
		this.branch = branch;
	}

	public String getIp() {
		return ip;
	}

	public void setIp(String ip) {
		this.ip = ip;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Employee other = (Employee) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	private static final long serialVersionUID = -6905483523393135835L;

}
