package br.com.inpe.ip.domain.service;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.persistence.NoResultException;

import org.hibernate.exception.ConstraintViolationException;

import br.com.inpe.ip.domain.entity.Group;
import br.com.inpe.ip.infrastructure.dao.GenericRepository;
import br.com.inpe.ip.infrastructure.dao.exception.BusinessException;

public class GroupService implements Serializable {
	
	@Inject GenericRepository<Group, Integer> repository;
	
	public void save (Group group) {
		try {
			repository.persist(group);
		} catch (ConstraintViolationException cve) {
			throw cve;
		}		
	}
	
	public void update (Group group) {
		try {
			repository.merge(group);
		} catch (ConstraintViolationException cve) {
			throw cve;
		}
	}
	
	public void remove (Group group) throws Exception {		
		repository.remove(group);			
	}
	
	public Group withId (Integer Id) {		
		return repository.find(Id);
	}
	
	public List<Group> findAll() {
		return repository.list();
	}
	
	public List<Group> searchBy (String name) throws BusinessException {				
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(Group.NAME, "%" +name.toUpperCase()+ "%");
		try {
			return repository.listByNamedQuery("Group.getByName", params);
		} catch ( NoResultException nre ) {
			throw new BusinessException("grupo não encontrado!");
		}
	}
		
	private static final long serialVersionUID = 8400576266914016141L;

}
